Attribute VB_Name = "Barcode"
Option Explicit

' The following functions are currently available:
'
'   1. GetPatternForCode25Interlieved - Returns a bit pattern for the 2 of 5 interleaved code
'   2. InitCode128                    - Initializes internal structures for code 128 encoding.
'                                       Must be called before calling any other code 128 functions
'   3. GetPatternForCodeUCC128        - Returns a bit pattern for EAN.UCC.128 code
'   4. GetPatternForCode128           - Returns a bit pattern for code 128 or EAN.UCC.128 code
'   5. CalculateMod10CheckDigit       - Returns a check digit (one character) using a standard
'                                       MOD10 algorithm
'   6. GetPatternForUPCA              - Returns a bit pattern for the UPC-A barcode
'   7. GetPatternForEAN13             - Returns a bit pattern for the EAN-13 barcode
'   8. GetPatternForCode39            - Returns a bit pattern for the 3 of 9 barcode
'   9. GetPatternForCodeSSCC18        - Returns a bit pattern for the SSCC-18 barcode
'
' See the sample application for the instructions on how to use these functions
'

Private Ptrn(0 To 106) As String
Private CodeA(0 To 106) As String
Private CodeB(0 To 106) As String
Private CodeC(0 To 106) As String
Private AsciiSpc(0 To 127) As String

Private Const CharsetA = 1
Private Const CharsetB = 2
Private Const CharsetC = 3

Private ASet As String
Private Const BSet = " !""#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_`abcdefghijklmnopqrstuvwxyz{|}~"
Private Const CSet = "0123456789"

Private CSumArray(0 To 128) As Integer
Private CSumArrayIdx As Integer

'''
''' For SSCC18 to work you need this function to return a correct GS1 code for your company
'''
Private Function GetGS1Code() As String
  GetGS1Code = "1234567"
End Function

' Warning!!! This function will correct the code by adding a check digit (if selected)
Public Function GetPatternForCode25Interlieved(ByRef InpStr As String, ByVal AddCheckDigit As Boolean) As String
    Dim ii As Integer
    Dim acc As Long

    GetPatternForCode25Interlieved = ""

    ' Trim spaces
    InpStr = Trim(InpStr)

    ' Check the allowable characters (digits only)
    For ii = 1 To Len(InpStr)
        If Not IsNumeric(Mid(InpStr, ii, 1)) Then Exit Function
    Next

    ' Check the length (output must have even number of digits)
    If AddCheckDigit And ((Len(InpStr) Mod 2) = 0) Then Exit Function
    If Not AddCheckDigit And ((Len(InpStr) Mod 2) = 1) Then Exit Function

    If AddCheckDigit Then
        ' Calculate Check Digit
        acc = 0
        For ii = 1 To Len(InpStr)
            If (ii Mod 2) = 1 Then
                acc = acc + CLng(Mid(InpStr, Len(InpStr) - ii + 1, 1)) * 3
            Else
                acc = acc + CLng(Mid(InpStr, Len(InpStr) - ii + 1, 1))
            End If
        Next
        InpStr = InpStr & CStr(10 - (acc Mod 10))
    End If

    ' Encode by pairs
    
    GetPatternForCode25Interlieved = "1010"
    ii = 1
    Do While ii < Len(InpStr)
        GetPatternForCode25Interlieved = GetPatternForCode25Interlieved & Encode25iPair(Mid(InpStr, ii, 1), Mid(InpStr, ii + 1, 1))
        ii = ii + 2
    Loop
    GetPatternForCode25Interlieved = GetPatternForCode25Interlieved & "11101"
End Function

Private Function Encode25iPair(ByVal D1 As Integer, ByVal D2 As Integer) As String
    Dim CodeArray(0 To 9) As String
    Dim sn As String, sp As String
    Dim ss As String
    Dim ii As Integer

    CodeArray(0) = "00110"
    CodeArray(1) = "10001"
    CodeArray(2) = "01001"
    CodeArray(3) = "11000"
    CodeArray(4) = "00101"
    CodeArray(5) = "10100"
    CodeArray(6) = "01100"
    CodeArray(7) = "00011"
    CodeArray(8) = "10010"
    CodeArray(9) = "01010"

    sp = CodeArray(D1)
    sn = CodeArray(D2)
    ss = ""
    
    For ii = 1 To 5
        If Mid(sp, ii, 1) = 0 Then
            ss = ss & "1"
        Else
            ss = ss & "111"
        End If
        If Mid(sn, ii, 1) = 0 Then
            ss = ss & "0"
        Else
            ss = ss & "000"
        End If
    Next
    Encode25iPair = ss
End Function

Public Sub InitCode128()
    Ptrn(0) = "11011001100"
    Ptrn(1) = "11001101100"
    Ptrn(2) = "11001100110"
    Ptrn(3) = "10010011000"
    Ptrn(4) = "10010001100"
    Ptrn(5) = "10001001100"
    Ptrn(6) = "10011001000"
    Ptrn(7) = "10011000100"
    Ptrn(8) = "10001100100"
    Ptrn(9) = "11001001000"
    Ptrn(10) = "11001000100"
    Ptrn(11) = "11000100100"
    Ptrn(12) = "10110011100"
    Ptrn(13) = "10011011100"
    Ptrn(14) = "10011001110"
    Ptrn(15) = "10111001100"
    Ptrn(16) = "10011101100"
    Ptrn(17) = "10011100110"
    Ptrn(18) = "11001110010"
    Ptrn(19) = "11001011100"
    Ptrn(20) = "11001001110"
    Ptrn(21) = "11011100100"
    Ptrn(22) = "11001110100"
    Ptrn(23) = "11101101110"
    Ptrn(24) = "11101001100"
    Ptrn(25) = "11100101100"
    Ptrn(26) = "11100100110"
    Ptrn(27) = "11101100100"
    Ptrn(28) = "11100110100"
    Ptrn(29) = "11100110010"
    Ptrn(30) = "11011011000"
    Ptrn(31) = "11011000110"
    Ptrn(32) = "11000110110"
    Ptrn(33) = "10100011000"
    Ptrn(34) = "10001011000"
    Ptrn(35) = "10001000110"
    Ptrn(36) = "10110001000"
    Ptrn(37) = "10001101000"
    Ptrn(38) = "10001100010"
    Ptrn(39) = "11010001000"
    Ptrn(40) = "11000101000"
    Ptrn(41) = "11000100010"
    Ptrn(42) = "10110111000"
    Ptrn(43) = "10110001110"
    Ptrn(44) = "10001101110"
    Ptrn(45) = "10111011000"
    Ptrn(46) = "10111000110"
    Ptrn(47) = "10001110110"
    Ptrn(48) = "11101110110"
    Ptrn(49) = "11010001110"
    Ptrn(50) = "11000101110"
    Ptrn(51) = "11011101000"
    Ptrn(52) = "11011100010"
    Ptrn(53) = "11011101110"
    Ptrn(54) = "11101011000"
    Ptrn(55) = "11101000110"
    Ptrn(56) = "11100010110"
    Ptrn(57) = "11101101000"
    Ptrn(58) = "11101100010"
    Ptrn(59) = "11100011010"
    Ptrn(60) = "11101111010"
    Ptrn(61) = "11001000010"
    Ptrn(62) = "11110001010"
    Ptrn(63) = "10100110000"
    Ptrn(64) = "10100001100"
    Ptrn(65) = "10010110000"
    Ptrn(66) = "10010000110"
    Ptrn(67) = "10000101100"
    Ptrn(68) = "10000100110"
    Ptrn(69) = "10110010000"
    Ptrn(70) = "10110000100"
    Ptrn(71) = "10011010000"
    Ptrn(72) = "10011000010"
    Ptrn(73) = "10000110100"
    Ptrn(74) = "10000110010"
    Ptrn(75) = "11000010010"
    Ptrn(76) = "11001010000"
    Ptrn(77) = "11110111010"
    Ptrn(78) = "11000010100"
    Ptrn(79) = "10001111010"
    Ptrn(80) = "10100111100"
    Ptrn(81) = "10010111100"
    Ptrn(82) = "10010011110"
    Ptrn(83) = "10111100100"
    Ptrn(84) = "10011110100"
    Ptrn(85) = "10011110010"
    Ptrn(86) = "11110100100"
    Ptrn(87) = "11110010100"
    Ptrn(88) = "11110010010"
    Ptrn(89) = "11011011110"
    Ptrn(90) = "11011110110"
    Ptrn(91) = "11110110110"
    Ptrn(92) = "10101111000"
    Ptrn(93) = "10100011110"
    Ptrn(94) = "10001011110"
    Ptrn(95) = "10111101000"
    Ptrn(96) = "10111100010"
    Ptrn(97) = "11110101000"
    Ptrn(98) = "11110100010"
    Ptrn(99) = "10111011110"
    Ptrn(100) = "10111101110"
    Ptrn(101) = "11101011110"
    Ptrn(102) = "11110101110"
    Ptrn(103) = "11010000100"
    Ptrn(104) = "11010010000"
    Ptrn(105) = "11010011100"
    Ptrn(106) = "1100011101011"
    
    CodeA(0) = " "
    CodeA(1) = "!"
    CodeA(2) = """"
    CodeA(3) = "#"
    CodeA(4) = "$"
    CodeA(5) = "%"
    CodeA(6) = "&"
    CodeA(7) = "'"
    CodeA(8) = "("
    CodeA(9) = ")"
    CodeA(10) = "*"
    CodeA(11) = "+"
    CodeA(12) = ","
    CodeA(13) = "-"
    CodeA(14) = "."
    CodeA(15) = "/"
    CodeA(16) = "0"
    CodeA(17) = "1"
    CodeA(18) = "2"
    CodeA(19) = "3"
    CodeA(20) = "4"
    CodeA(21) = "5"
    CodeA(22) = "6"
    CodeA(23) = "7"
    CodeA(24) = "8"
    CodeA(25) = "9"
    CodeA(26) = ":"
    CodeA(27) = ";"
    CodeA(28) = "<"
    CodeA(29) = "="
    CodeA(30) = ">"
    CodeA(31) = "?"
    CodeA(32) = "@"
    CodeA(33) = "A"
    CodeA(34) = "B"
    CodeA(35) = "C"
    CodeA(36) = "D"
    CodeA(37) = "E"
    CodeA(38) = "F"
    CodeA(39) = "G"
    CodeA(40) = "H"
    CodeA(41) = "I"
    CodeA(42) = "J"
    CodeA(43) = "K"
    CodeA(44) = "L"
    CodeA(45) = "M"
    CodeA(46) = "N"
    CodeA(47) = "O"
    CodeA(48) = "P"
    CodeA(49) = "Q"
    CodeA(50) = "R"
    CodeA(51) = "S"
    CodeA(52) = "T"
    CodeA(53) = "U"
    CodeA(54) = "V"
    CodeA(55) = "W"
    CodeA(56) = "X"
    CodeA(57) = "Y"
    CodeA(58) = "Z"
    CodeA(59) = "["
    CodeA(60) = "\"
    CodeA(61) = "]"
    CodeA(62) = "^"
    CodeA(63) = "_"
    CodeA(64) = "nul"
    CodeA(65) = "soh"
    CodeA(66) = "stx"
    CodeA(67) = "etx"
    CodeA(68) = "eot"
    CodeA(69) = "eno"
    CodeA(70) = "ack"
    CodeA(71) = "bel"
    CodeA(72) = "bs"
    CodeA(73) = "ht"
    CodeA(74) = "lf"
    CodeA(75) = "vt"
    CodeA(76) = "ff"
    CodeA(77) = "cr"
    CodeA(78) = "s0"
    CodeA(79) = "s1"
    CodeA(80) = "dle"
    CodeA(81) = "dc1"
    CodeA(82) = "dc2"
    CodeA(83) = "dc3"
    CodeA(84) = "dc4"
    CodeA(85) = "nak"
    CodeA(86) = "syn"
    CodeA(87) = "etb"
    CodeA(88) = "can"
    CodeA(89) = "em"
    CodeA(90) = "sub"
    CodeA(91) = "esc"
    CodeA(92) = "fs"
    CodeA(93) = "gs"
    CodeA(94) = "rs"
    CodeA(95) = "us"
    CodeA(96) = "Fnc3"
    CodeA(97) = "Fnc2"
    CodeA(98) = "Shift"
    CodeA(99) = "CodeC"
    CodeA(100) = "CodeB"
    CodeA(101) = "Fnc4"
    CodeA(102) = "Fnc1"
    CodeA(103) = "StartA"
    CodeA(104) = "StartB"
    CodeA(105) = "StartC"
    CodeA(106) = "Stop"
    
    CodeB(0) = " "
    CodeB(1) = "!"
    CodeB(2) = """"
    CodeB(3) = "#"
    CodeB(4) = "$"
    CodeB(5) = "%"
    CodeB(6) = "&"
    CodeB(7) = "'"
    CodeB(8) = "("
    CodeB(9) = ")"
    CodeB(10) = "*"
    CodeB(11) = "+"
    CodeB(12) = ","
    CodeB(13) = "-"
    CodeB(14) = "."
    CodeB(15) = "/"
    CodeB(16) = "0"
    CodeB(17) = "1"
    CodeB(18) = "2"
    CodeB(19) = "3"
    CodeB(20) = "4"
    CodeB(21) = "5"
    CodeB(22) = "6"
    CodeB(23) = "7"
    CodeB(24) = "8"
    CodeB(25) = "9"
    CodeB(26) = ":"
    CodeB(27) = ";"
    CodeB(28) = "<"
    CodeB(29) = "="
    CodeB(30) = ">"
    CodeB(31) = "?"
    CodeB(32) = "@"
    CodeB(33) = "A"
    CodeB(34) = "B"
    CodeB(35) = "C"
    CodeB(36) = "D"
    CodeB(37) = "E"
    CodeB(38) = "F"
    CodeB(39) = "G"
    CodeB(40) = "H"
    CodeB(41) = "I"
    CodeB(42) = "J"
    CodeB(43) = "K"
    CodeB(44) = "L"
    CodeB(45) = "M"
    CodeB(46) = "N"
    CodeB(47) = "O"
    CodeB(48) = "P"
    CodeB(49) = "Q"
    CodeB(50) = "R"
    CodeB(51) = "S"
    CodeB(52) = "T"
    CodeB(53) = "U"
    CodeB(54) = "V"
    CodeB(55) = "W"
    CodeB(56) = "X"
    CodeB(57) = "Y"
    CodeB(58) = "Z"
    CodeB(59) = "["
    CodeB(60) = "\"
    CodeB(61) = "]"
    CodeB(62) = "^"
    CodeB(63) = "_"
    CodeB(64) = "`"
    CodeB(65) = "a"
    CodeB(66) = "b"
    CodeB(67) = "c"
    CodeB(68) = "d"
    CodeB(69) = "e"
    CodeB(70) = "f"
    CodeB(71) = "g"
    CodeB(72) = "h"
    CodeB(73) = "i"
    CodeB(74) = "j"
    CodeB(75) = "k"
    CodeB(76) = "l"
    CodeB(77) = "m"
    CodeB(78) = "n"
    CodeB(79) = "o"
    CodeB(80) = "p"
    CodeB(81) = "q"
    CodeB(82) = "r"
    CodeB(83) = "s"
    CodeB(84) = "t"
    CodeB(85) = "u"
    CodeB(86) = "v"
    CodeB(87) = "w"
    CodeB(88) = "x"
    CodeB(89) = "y"
    CodeB(90) = "z"
    CodeB(91) = "{"
    CodeB(92) = "|"
    CodeB(93) = "}"
    CodeB(94) = "~"
    CodeB(95) = "del"
    CodeB(96) = "Fnc3"
    CodeB(97) = "Fnc2"
    CodeB(98) = "Shift"
    CodeB(99) = "CodeC"
    CodeB(100) = "Fnc4"
    CodeB(101) = "CodeA"
    CodeB(102) = "Fnc1"
    CodeB(103) = "StartA"
    CodeB(104) = "StartB"
    CodeB(105) = "StartC"
    CodeB(106) = "Stop"
    
    CodeC(0) = "00"
    CodeC(1) = "01"
    CodeC(2) = "02"
    CodeC(3) = "03"
    CodeC(4) = "04"
    CodeC(5) = "05"
    CodeC(6) = "06"
    CodeC(7) = "07"
    CodeC(8) = "08"
    CodeC(9) = "09"
    CodeC(10) = "10"
    CodeC(11) = "11"
    CodeC(12) = "12"
    CodeC(13) = "13"
    CodeC(14) = "14"
    CodeC(15) = "15"
    CodeC(16) = "16"
    CodeC(17) = "17"
    CodeC(18) = "18"
    CodeC(19) = "19"
    CodeC(20) = "20"
    CodeC(21) = "21"
    CodeC(22) = "22"
    CodeC(23) = "23"
    CodeC(24) = "24"
    CodeC(25) = "25"
    CodeC(26) = "26"
    CodeC(27) = "27"
    CodeC(28) = "28"
    CodeC(29) = "29"
    CodeC(30) = "30"
    CodeC(31) = "31"
    CodeC(32) = "32"
    CodeC(33) = "33"
    CodeC(34) = "34"
    CodeC(35) = "35"
    CodeC(36) = "36"
    CodeC(37) = "37"
    CodeC(38) = "38"
    CodeC(39) = "39"
    CodeC(40) = "40"
    CodeC(41) = "41"
    CodeC(42) = "42"
    CodeC(43) = "43"
    CodeC(44) = "44"
    CodeC(45) = "45"
    CodeC(46) = "46"
    CodeC(47) = "47"
    CodeC(48) = "48"
    CodeC(49) = "49"
    CodeC(50) = "50"
    CodeC(51) = "51"
    CodeC(52) = "52"
    CodeC(53) = "53"
    CodeC(54) = "54"
    CodeC(55) = "55"
    CodeC(56) = "56"
    CodeC(57) = "57"
    CodeC(58) = "58"
    CodeC(59) = "59"
    CodeC(60) = "60"
    CodeC(61) = "61"
    CodeC(62) = "62"
    CodeC(63) = "63"
    CodeC(64) = "64"
    CodeC(65) = "65"
    CodeC(66) = "66"
    CodeC(67) = "67"
    CodeC(68) = "68"
    CodeC(69) = "69"
    CodeC(70) = "70"
    CodeC(71) = "71"
    CodeC(72) = "72"
    CodeC(73) = "73"
    CodeC(74) = "74"
    CodeC(75) = "75"
    CodeC(76) = "76"
    CodeC(77) = "77"
    CodeC(78) = "78"
    CodeC(79) = "79"
    CodeC(80) = "80"
    CodeC(81) = "81"
    CodeC(82) = "82"
    CodeC(83) = "83"
    CodeC(84) = "84"
    CodeC(85) = "85"
    CodeC(86) = "86"
    CodeC(87) = "87"
    CodeC(88) = "88"
    CodeC(89) = "89"
    CodeC(90) = "90"
    CodeC(91) = "91"
    CodeC(92) = "92"
    CodeC(93) = "93"
    CodeC(94) = "94"
    CodeC(95) = "95"
    CodeC(96) = "96"
    CodeC(97) = "97"
    CodeC(98) = "98"
    CodeC(99) = "99"
    CodeC(100) = "CodeB"
    CodeC(101) = "CodeA"
    CodeC(102) = "Fnc1"
    CodeC(103) = "StartA"
    CodeC(104) = "StartB"
    CodeC(105) = "StartC"
    CodeC(106) = "Stop"
    
    ASet = " !""#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_" & _
           Chr(0) & Chr(1) & Chr(2) & Chr(3) & Chr(4) & Chr(5) & Chr(6) & Chr(7) & _
           Chr(8) & Chr(9) & Chr(10) & Chr(11) & Chr(12) & Chr(13) & Chr(14) & Chr(15) & _
           Chr(16) & Chr(17) & Chr(18) & Chr(19) & Chr(20) & Chr(21) & Chr(22) & Chr(23) & _
           Chr(24) & Chr(25) & Chr(26) & Chr(27) & Chr(28) & Chr(29) & Chr(30) & Chr(31)
End Sub

Private Sub InitAsciiSpcTable()
    Dim ii As Integer
    For ii = 32 To 126
        AsciiSpc(ii) = ""
    Next
    AsciiSpc(0) = "NUL"
    AsciiSpc(1) = "SOH"
    AsciiSpc(2) = "STX"
    AsciiSpc(3) = "ETX"
    AsciiSpc(4) = "EOT"
    AsciiSpc(5) = "ENQ"
    AsciiSpc(6) = "ACK"
    AsciiSpc(7) = "BEL"
    AsciiSpc(8) = "BS"
    AsciiSpc(9) = "HT"
    AsciiSpc(10) = "LF"
    AsciiSpc(11) = "VT"
    AsciiSpc(12) = "FF"
    AsciiSpc(13) = "CR"
    AsciiSpc(14) = "SO"
    AsciiSpc(15) = "SI"
    AsciiSpc(16) = "DLE"
    AsciiSpc(17) = "DC1"
    AsciiSpc(18) = "DC2"
    AsciiSpc(19) = "DC3"
    AsciiSpc(20) = "DC4"
    AsciiSpc(21) = "NAK"
    AsciiSpc(22) = "SYN"
    AsciiSpc(23) = "ETB"
    AsciiSpc(24) = "CAN"
    AsciiSpc(25) = "EM"
    AsciiSpc(26) = "SUB"
    AsciiSpc(27) = "ESC"
    AsciiSpc(28) = "FS"
    AsciiSpc(29) = "GS"
    AsciiSpc(30) = "RS"
    AsciiSpc(31) = "US"
    AsciiSpc(127) = "DEL"
End Sub

Private Function DetectCharset(InpStr As String) As Integer
    ' Basically, for most purposes charset B should be used unless there are a mostly digits in the
    ' input string. Charset A contains a lot of rarely used special characters and as such will be rarely
    ' used too. So, we start with charset B unless we see at least 6 consecutive digits at the beginning.
    Dim ii As Integer, ncnt As Integer, xcnt As Integer
    DetectCharset = CharsetB
    If Len(InpStr) = 0 Then Exit Function
    ncnt = 0
    xcnt = 0
    For ii = 1 To Len(InpStr)
        If Asc(Mid(InpStr, 1, 1)) >= 32 Then
            xcnt = 1
            Exit For
        End If
        ncnt = ncnt + 1
    Next
    If (ncnt > 5) Or ((ncnt > 0) And (xcnt = 0)) Then
        DetectCharset = CharsetA
        Exit Function
    End If
    ncnt = 0
    xcnt = 0
    For ii = 1 To Len(InpStr)
        If InStr(CSet, Mid(InpStr, ii, 1)) = 0 Then
            xcnt = 1
            Exit For
        End If
        ncnt = ncnt + 1
    Next
    If (ncnt > 5) Or ((ncnt > 0) And (xcnt = 0)) Then
        DetectCharset = CharsetC
    End If
End Function

Public Function GetPatternForCodeSSCC18(ByVal SerialNumber As String, ByRef strOut As String) As String
    Dim AppPrefix As String
    Dim ExtensionDigit As String
    Dim GS1Code As String
    Dim CheckDigit As String
    
    AppPrefix = "00"
    ExtensionDigit = "0"
    GS1Code = GetGS1Code()
    strOut = AppPrefix & ExtensionDigit & GS1Code & right("000000000" & SerialNumber, 9)
    CheckDigit = CalculateMod10CheckDigit(strOut)
    strOut = strOut & CheckDigit
    GetPatternForCodeSSCC18 = GetPatternForCode128(strOut, False)
End Function

Public Function GetPatternForCodeUCC128(ByVal InpStr As String) As String
    GetPatternForCodeUCC128 = GetPatternForCode128(InpStr, True)
End Function

Private Function Get128Pattern(ByVal Charset As Integer, ByVal Name As String) As String
    Dim ii As Integer
    Dim ss As String
    
    Get128Pattern = ""
    Select Case Charset
        Case CharsetA:
            For ii = 0 To 106
                If CodeA(ii) = Name Then
                    Get128Pattern = Ptrn(ii)
                    AddToCSumArray (ii)
                    Exit Function
                End If
            Next
        Case CharsetB:
            For ii = 0 To 106
                If CodeB(ii) = Name Then
                    Get128Pattern = Ptrn(ii)
                    AddToCSumArray (ii)
                    Exit Function
                End If
            Next
        Case CharsetC:
            For ii = 0 To 106
                If CodeC(ii) = Name Then
                    Get128Pattern = Ptrn(ii)
                    AddToCSumArray (ii)
                    Exit Function
                End If
            Next
    End Select
    ' If we are here then our input character did not match to the current
    ' character set.
    Select Case Charset
        Case CharsetA:
            If InStr(BSet, Name) > 0 Then
                Get128Pattern = Get128Pattern(CharsetA, "Shift") & Get128Pattern(CharsetB, Name)
            End If
        Case CharsetB:
            If InStr(ASet, Name) > 0 Then
                Get128Pattern = Get128Pattern(CharsetB, "Shift") & Get128Pattern(CharsetA, Name)
            Else
                For ii = 0 To 106
                    If CodeA(ii) = Name Then
                        ss = Get128Pattern(CharsetB, "Shift") & Ptrn(ii)
                        Get128Pattern = ss
                        AddToCSumArray (ii)
                        Exit For
                    End If
                Next
            End If
        Case CharsetC:
            If InStr(BSet, Name) > 0 Then
                ' We have to do this because the execution order is not guaranteed
                ss = Get128Pattern(CharsetC, "CodeB")
                ss = ss & Get128Pattern(CharsetB, Name)
                Get128Pattern = ss
            ElseIf InStr(ASet, Name) > 0 Then
                ' We have to do this because the execution order is not guaranteed
                ss = Get128Pattern(CharsetC, "CodeA")
                ss = ss & Get128Pattern(CharsetA, Name)
                Get128Pattern = ss
            Else
                For ii = 0 To 106
                    If CodeA(ii) = Name Then
                        ss = Get128Pattern(CharsetC, "CodeA") & Ptrn(ii)
                        Get128Pattern = ss
                        AddToCSumArray (ii)
                        Exit For
                    End If
                Next
            End If
    End Select
End Function

Private Sub AddToCSumArray(CIdx As Integer)
    CSumArray(CSumArrayIdx) = CIdx
    CSumArrayIdx = CSumArrayIdx + 1
End Sub

Public Function GetPatternForCode128(ByVal InpStr As String, UccOutput As Boolean) As String
    Dim strOut As String
    Dim UseCharset As Integer
    Dim ii As Integer, tcs As Integer
    Dim CSum As Long
    
    GetPatternForCode128 = ""
    strOut = ""
    CSumArrayIdx = 0
    For ii = 0 To 128
        CSumArray(ii) = 0
    Next
    
    UseCharset = DetectCharset(InpStr)
    Select Case UseCharset
        Case CharsetA:
            strOut = Get128Pattern(UseCharset, "StartA")
        Case CharsetB:
            strOut = Get128Pattern(UseCharset, "StartB")
        Case CharsetC:
            strOut = Get128Pattern(UseCharset, "StartC")
    End Select
    If UccOutput Then strOut = strOut & Get128Pattern(UseCharset, "Fnc1")
    
    ii = 1
    Do While ii <= Len(InpStr)
        tcs = DetectCharset(Mid(InpStr, ii)) ' See if we might have to swich to a different charset
        If UseCharset <> tcs Then
            ' Switch the charset
            Select Case tcs
                Case CharsetA:
                    strOut = strOut & Get128Pattern(UseCharset, "CodeA")
                Case CharsetB:
                    strOut = strOut & Get128Pattern(UseCharset, "CodeB")
                Case CharsetC:
                    strOut = strOut & Get128Pattern(UseCharset, "CodeC")
            End Select
        End If
        UseCharset = tcs
        If UseCharset = CharsetC Then
            If ii = Len(InpStr) Then
                ' Not enough digits left to complete the last pair
                ' Switching to CharsetB
                strOut = strOut & Get128Pattern(UseCharset, "CodeB")
                strOut = strOut & Get128Pattern(CharsetB, Mid(InpStr, ii, 1))
                ii = ii + 1
            Else
                strOut = strOut & Get128Pattern(UseCharset, Mid(InpStr, ii, 2))
                ii = ii + 2
            End If
        Else
            strOut = strOut & Get128Pattern(UseCharset, Mid(InpStr, ii, 1))
            ii = ii + 1
        End If
    Loop
    
    ' We are done with encoding. Add a check digit
    CSum = CSumArray(0)
    For ii = 1 To CSumArrayIdx
        CSum = CSum + (CSumArray(ii) * ii)
    Next
    ii = CSum Mod 103
    strOut = strOut & Ptrn(ii)
    
    ' Add a Stop character
    strOut = strOut & Ptrn(106)
    
    GetPatternForCode128 = strOut
End Function

Public Function CalculateMod10CheckDigit(ByVal InpStr As String) As String
    Dim acc As Integer
    Dim ii As Integer

    CalculateMod10CheckDigit = ""
    acc = 0
    For ii = 1 To Len(InpStr)
        If InStr(CSet, Mid(InpStr, Len(InpStr) - ii + 1, 1)) = 0 Then Exit Function
        If (ii Mod 2) = 1 Then
            acc = acc + CLng(Mid(InpStr, Len(InpStr) - ii + 1, 1)) * 3
        Else
            acc = acc + CLng(Mid(InpStr, Len(InpStr) - ii + 1, 1))
        End If
    Next
    If (acc Mod 10) = 0 Then
        CalculateMod10CheckDigit = "0"
    Else
        CalculateMod10CheckDigit = CStr(10 - (acc Mod 10))
    End If
End Function

Public Function GetPatternForCode39(ByVal InpStr As String) As String
    Dim ii As Long
    Dim ss As String

    ii = InStr(InpStr, "*")
    Do While ii > 0
        InpStr = Mid(InpStr, 1, ii - 1) & Mid(InpStr, ii + 1)
        ii = InStr(InpStr, "*")
    Loop
    InpStr = "*" & InpStr & "*"
    
    ss = ""
    
    For ii = 1 To Len(InpStr)
        Select Case Mid(InpStr, ii, 1)
            Case "*":
                ss = ss & "100010111011101"
            Case "-":
                ss = ss & "100010101110111"
            Case "$":
                ss = ss & "100010001000101"
            Case "%":
                ss = ss & "101000100010001"
            Case ".":
                ss = ss & "111000101011101"
            Case "/":
                ss = ss & "100010001010001"
            Case "+":
                ss = ss & "100010100010001"
            Case "0":
                ss = ss & "101000111011101"
            Case "1":
                ss = ss & "111010001010111"
            Case "2":
                ss = ss & "101110001010111"
            Case "3":
                ss = ss & "111011100010101"
            Case "4":
                ss = ss & "101000111010111"
            Case "5":
                ss = ss & "111010001110101"
            Case "6":
                ss = ss & "101110001110101"
            Case "7":
                ss = ss & "101000101110111"
            Case "8":
                ss = ss & "111010001011101"
            Case "9":
                ss = ss & "101110001011101"
            Case "A":
                ss = ss & "111010100010111"
            Case "B":
                ss = ss & "101110100010111"
            Case "C":
                ss = ss & "111011101000101"
            Case "D":
                ss = ss & "101011100010111"
            Case "E":
                ss = ss & "111010111000101"
            Case "F":
                ss = ss & "101110111000101"
            Case "G":
                ss = ss & "101010001110111"
            Case "H":
                ss = ss & "111010100011101"
            Case "I":
                ss = ss & "101110100011101"
            Case "J":
                ss = ss & "101011100011101"
            Case "K":
                ss = ss & "111010101000111"
            Case "L":
                ss = ss & "101110101000111"
            Case "M":
                ss = ss & "111011101010001"
            Case "N":
                ss = ss & "101011101000111"
            Case "O":
                ss = ss & "111010111010001"
            Case "P":
                ss = ss & "101110111010001"
            Case "Q":
                ss = ss & "101010111000111"
            Case "R":
                ss = ss & "111010101110001"
            Case "S":
                ss = ss & "101110101110001"
            Case "T":
                ss = ss & "101011101110001"
            Case "U":
                ss = ss & "111000101010111"
            Case "V":
                ss = ss & "100011101010111"
            Case "W":
                ss = ss & "111000111010101"
            Case "X":
                ss = ss & "100010111010111"
            Case "Y":
                ss = ss & "111000101110101"
            Case "Z":
                ss = ss & "100011101110101"
            Case Else:
                ss = ss & "100011101011101"
        End Select
        ss = ss & "0"
    Next
    
    GetPatternForCode39 = ss
End Function

Private Function UPCTableA(ByVal N As Integer) As String
    Select Case N
        Case 0:
            UPCTableA = "0001101"
        Case 1:
            UPCTableA = "0011001"
        Case 2:
            UPCTableA = "0010011"
        Case 3:
            UPCTableA = "0111101"
        Case 4:
            UPCTableA = "0100011"
        Case 5:
            UPCTableA = "0110001"
        Case 6:
            UPCTableA = "0101111"
        Case 7:
            UPCTableA = "0111011"
        Case 8:
            UPCTableA = "0110111"
        Case 9:
            UPCTableA = "0001011"
    End Select
End Function

Private Function UPCTableB(ByVal N As Integer) As String
    Select Case N
        Case 0:
            UPCTableB = "0100111"
        Case 1:
            UPCTableB = "0110011"
        Case 2:
            UPCTableB = "0011011"
        Case 3:
            UPCTableB = "0100001"
        Case 4:
            UPCTableB = "0011101"
        Case 5:
            UPCTableB = "0111001"
        Case 6:
            UPCTableB = "0000101"
        Case 7:
            UPCTableB = "0010001"
        Case 8:
            UPCTableB = "0001001"
        Case 9:
            UPCTableB = "0010111"
    End Select
End Function

Private Function UPCTableC(ByVal N As Integer) As String
    Select Case N
        Case 0:
            UPCTableC = "1110010"
        Case 1:
            UPCTableC = "1100110"
        Case 2:
            UPCTableC = "1101100"
        Case 3:
            UPCTableC = "1000010"
        Case 4:
            UPCTableC = "1011100"
        Case 5:
            UPCTableC = "1001110"
        Case 6:
            UPCTableC = "1010000"
        Case 7:
            UPCTableC = "1000100"
        Case 8:
            UPCTableC = "1001000"
        Case 9:
            UPCTableC = "1110100"
    End Select
End Function

Private Function UPCStartStop() As String
    UPCStartStop = "101"
End Function

Private Function UPCDelimiter() As String
    UPCDelimiter = "01010"
End Function

Public Function GetPatternForEAN13(ByVal Code As String) As String
    Dim H1 As String, H2 As String
    
    GetPatternForEAN13 = ""
    If Len(Code) <> 13 Then Exit Function
    
    Select Case Mid(Code, 1, 1)
        Case "0":
            H1 = UPCTableA(Mid(Code, 2, 1)) & UPCTableA(Mid(Code, 3, 1)) & UPCTableA(Mid(Code, 4, 1)) & UPCTableA(Mid(Code, 5, 1)) & UPCTableA(Mid(Code, 6, 1)) & UPCTableA(Mid(Code, 7, 1))
        Case "1":
            H1 = UPCTableA(Mid(Code, 2, 1)) & UPCTableA(Mid(Code, 3, 1)) & UPCTableB(Mid(Code, 4, 1)) & UPCTableA(Mid(Code, 5, 1)) & UPCTableB(Mid(Code, 6, 1)) & UPCTableB(Mid(Code, 7, 1))
        Case "2":
            H1 = UPCTableA(Mid(Code, 2, 1)) & UPCTableA(Mid(Code, 3, 1)) & UPCTableB(Mid(Code, 4, 1)) & UPCTableB(Mid(Code, 5, 1)) & UPCTableA(Mid(Code, 6, 1)) & UPCTableB(Mid(Code, 7, 1))
        Case "3":
            H1 = UPCTableA(Mid(Code, 2, 1)) & UPCTableA(Mid(Code, 3, 1)) & UPCTableB(Mid(Code, 4, 1)) & UPCTableB(Mid(Code, 5, 1)) & UPCTableB(Mid(Code, 6, 1)) & UPCTableA(Mid(Code, 7, 1))
        Case "4":
            H1 = UPCTableA(Mid(Code, 2, 1)) & UPCTableB(Mid(Code, 3, 1)) & UPCTableA(Mid(Code, 4, 1)) & UPCTableA(Mid(Code, 5, 1)) & UPCTableB(Mid(Code, 6, 1)) & UPCTableB(Mid(Code, 7, 1))
        Case "5":
            H1 = UPCTableA(Mid(Code, 2, 1)) & UPCTableB(Mid(Code, 3, 1)) & UPCTableB(Mid(Code, 4, 1)) & UPCTableA(Mid(Code, 5, 1)) & UPCTableA(Mid(Code, 6, 1)) & UPCTableB(Mid(Code, 7, 1))
        Case "6":
            H1 = UPCTableA(Mid(Code, 2, 1)) & UPCTableB(Mid(Code, 3, 1)) & UPCTableB(Mid(Code, 4, 1)) & UPCTableB(Mid(Code, 5, 1)) & UPCTableA(Mid(Code, 6, 1)) & UPCTableA(Mid(Code, 7, 1))
        Case "7":
            H1 = UPCTableA(Mid(Code, 2, 1)) & UPCTableB(Mid(Code, 3, 1)) & UPCTableA(Mid(Code, 4, 1)) & UPCTableB(Mid(Code, 5, 1)) & UPCTableA(Mid(Code, 6, 1)) & UPCTableB(Mid(Code, 7, 1))
        Case "8":
            H1 = UPCTableA(Mid(Code, 2, 1)) & UPCTableB(Mid(Code, 3, 1)) & UPCTableA(Mid(Code, 4, 1)) & UPCTableB(Mid(Code, 5, 1)) & UPCTableB(Mid(Code, 6, 1)) & UPCTableA(Mid(Code, 7, 1))
        Case "9":
            H1 = UPCTableA(Mid(Code, 2, 1)) & UPCTableB(Mid(Code, 3, 1)) & UPCTableB(Mid(Code, 4, 1)) & UPCTableA(Mid(Code, 5, 1)) & UPCTableB(Mid(Code, 6, 1)) & UPCTableA(Mid(Code, 7, 1))
    End Select
    H2 = UPCTableC(Mid(Code, 8, 1)) & UPCTableC(Mid(Code, 9, 1)) & UPCTableC(Mid(Code, 10, 1)) & UPCTableC(Mid(Code, 11, 1)) & UPCTableC(Mid(Code, 12, 1)) & UPCTableC(Mid(Code, 13, 1))
    GetPatternForEAN13 = UPCStartStop & H1 & UPCDelimiter & H2 & UPCStartStop
End Function

Public Function GetPatternForUPCA(ByVal Code As String) As String
    Dim H1 As String, H2 As String
    
    GetPatternForUPCA = ""
    If Len(Code) <> 12 Then Exit Function
    
    H1 = UPCTableA(Mid(Code, 1, 1)) & UPCTableA(Mid(Code, 2, 1)) & UPCTableA(Mid(Code, 3, 1)) & UPCTableA(Mid(Code, 4, 1)) & UPCTableA(Mid(Code, 5, 1)) & UPCTableA(Mid(Code, 6, 1))
    H2 = UPCTableC(Mid(Code, 7, 1)) & UPCTableC(Mid(Code, 8, 1)) & UPCTableC(Mid(Code, 9, 1)) & UPCTableC(Mid(Code, 10, 1)) & UPCTableC(Mid(Code, 11, 1)) & UPCTableC(Mid(Code, 12, 1))
    GetPatternForUPCA = UPCStartStop & H1 & UPCDelimiter & H2 & UPCStartStop
End Function

